%{
/*
   Hoarescope - Use Hoare logic for program correctness proofs
   Copyright (C) 2004 Markus Triska

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA

   Contact me at triska@gmx.at
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "node.h"
#include "parser.h"

extern int yyparse(void);
extern int yyerror(char *);

%}
%%
"if"         { return IF; }
"then"       { return THEN; }
"else"       { return ELSETOK; }
"while"      { return WHILE; }
"do"         { return DO; }
"begin"      { return BEGINTOK; }
"end"        { return ENDTOK; }
">="         { return GE; }
"<="         { return LE; }
"="          { return EQ; }
"!="         { return NE; }
":="         { return ASSIGN; }
"&&"         { return AND; }
"->"         { return IMPLIES; }
"||"         { return OR; }
"!"          { return NOT; }
"WHERE"      { return WHERE; }
"IS"         { return IS; }



#.*\n       { /* skip comment */ }

[a-z]+[0-9]* {
		// printf("Token: LOWCASESTRING\n");
		yylval.tok_string = (char *) malloc(strlen(yytext) + 1);
		if (yylval.tok_string == NULL) {
			yyerror("lexer out of memory");
		}
		strcpy(yylval.tok_string, yytext);
		return LOWCASESTRING;
	}

[A-Z]+[0-9]* {
		// printf("Token: UPCASESTRING\n");
		yylval.tok_string = (char *) malloc(strlen(yytext) + 1);
		if (yylval.tok_string == NULL) {
			yyerror("lexer out of memory");
		}
		strcpy(yylval.tok_string, yytext);
		return UPCASESTRING;
	}

\?       { return EXISTS; }

\%       { return FORALL; }


[0-9]+ {
		// printf("Token: NUMBER\n");
		yylval.tok_ivalue = atoi(yytext);
		return NUMBER;
	}

[ \r\t] {	// printf("Token: WHITESPACE\n");
	}

\n	{ return EOL; }

.       { return *yytext; }
%%
int yywrap() { return 1; }
